@@ -21,6 +21,9 @@
 """
 
 import os, time, codecs, base64
+import hashlib
+import hmac
+from copy import deepcopy
 import md5crypt
 
 try:
@@ -31,13 +34,15 @@ except ImportError:
 from MoinMoin import log
 logging = log.getLogger(__name__)
 
-from MoinMoin.support.python_compatibility import hash_new, hmac_new
-
 from MoinMoin import config, caching, wikiutil, i18n, events
 from werkzeug.security import safe_str_cmp as safe_str_equal
 from MoinMoin.util import timefuncs, random_string
 from MoinMoin.wikiutil import url_quote_plus
 
+# for efficient lookup <attr> -> userid, we keep an index of this in the cache.
+# the attribute names in here should be uniquely identifying a user.
+CACHED_USER_ATTRS = ['name', 'email', 'jid', 'openids', ]
+
 
 def getUserList(request):
     """ Get a list of all (numerical) user IDs.
@@ -53,7 +58,10 @@ def getUserList(request):
     return userlist
 
 def get_by_filter(request, filter_func):
-    """ Searches for an user with a given filter function """
+    """ Searches for a user with a given filter function
+
+    Be careful: SLOW for big wikis, rather use _getUserIdByKey & related.
+    """
     for uid in getUserList(request):
         theuser = User(request, uid)
         if filter_func(theuser):
@@ -61,15 +69,17 @@ def get_by_filter(request, filter_func):
 
 def get_by_email_address(request, email_address):
     """ Searches for an user with a particular e-mail address and returns it. """
-    filter_func = lambda user: user.valid and user.email.lower() == email_address.lower()
-    return get_by_filter(request, filter_func)
+    uid = _getUserIdByKey(request, 'email', email_address, case=False)
+    if uid is not None:
+        return User(request, uid)
 
 def get_by_jabber_id(request, jabber_id):
     """ Searches for an user with a perticular jabber id and returns it. """
-    filter_func = lambda user: user.valid and user.jid.lower() == jabber_id.lower()
-    return get_by_filter(request, filter_func)
+    uid = _getUserIdByKey(request, 'jid', jabber_id, case=False)
+    if uid is not None:
+        return User(request, uid)
 
-def _getUserIdByKey(request, key, search):
+def _getUserIdByKey(request, key, search, case=True):
     """ Get the user ID for a specified key/value pair.
 
     This method must only be called for keys that are
@@ -77,43 +87,135 @@ def _getUserIdByKey(request, key, search):
 
     @param key: the key to look in
     @param search: the value to look for
+    @param case: do a case-sensitive lookup?
     @return the corresponding user ID or None
     """
-    if not search or not key:
+    if key not in CACHED_USER_ATTRS:
+        raise ValueError("unsupported key, must be in CACHED_USER_ATTRS")
+    if not search:
         return None
+    cfg_cache_attr = key + "2id"
+    if not case:
+        cfg_cache_attr += "_lower"
+        search = search.lower()
     cfg = request.cfg
-    cachekey = '%s2id' % key
     try:
-        _key2id = getattr(cfg.cache, cachekey)
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        from_disk = False
     except AttributeError:
-        arena = 'user'
-        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
-        try:
-            _key2id = cache.content()
-        except caching.CacheError:
-            _key2id = {}
-        setattr(cfg.cache, cachekey, _key2id)
-    uid = _key2id.get(search, None)
+        # no in-memory cache there - initialize it / load it from disk
+        loadLookupCaches(request)
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        from_disk = True  # we just loaded the stuff from disk
+    uid = attr2id.get(search, None)
+    if uid is None and not from_disk:
+        # we do not have the entry we searched for.
+        # we didn't find it in some in-memory cache, try refreshing these from disk
+        loadLookupCaches(request)
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        from_disk = True  # we just loaded the stuff from disk
+        uid = attr2id.get(search, None)
     if uid is None:
-        for userid in getUserList(request):
-            u = User(request, id=userid)
-            if hasattr(u, key):
-                value = getattr(u, key)
-                if isinstance(value, list):
-                    for val in value:
-                        _key2id[val] = userid
-                else:
-                    _key2id[value] = userid
-        arena = 'user'
-        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
-        try:
-            cache.update(_key2id)
-        except caching.CacheError:
-            pass
-        uid = _key2id.get(search, None)
+        # we do not have the entry we searched for.
+        # we don't have it in the on-disk cache, cache MISS.
+        # could be because:
+        # a) ok: we have no such search value in the profiles
+        # b) fault: the cache is incoherent with the profiles
+        # c) fault: reading the cache from disk failed, due to an error
+        # d) ok: same as c), but just because no ondisk cache has been built yet
+        rebuildLookupCaches(request)  # XXX expensive
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        uid = attr2id.get(search, None)
     return uid
 
 
+def setMemoryLookupCaches(request, cache):
+    """set the in-memory cache from the given cache contents
+
+    @param request: the request object
+    @param cache: either a dict of attrname -> attrcache to set the in-memory cache,
+                  or None to delete the in-memory cache.
+    """
+    for attrname in CACHED_USER_ATTRS:
+        if cache is None:
+            try:
+                delattr(request.cfg.cache, attrname + "2id")
+            except:
+                pass
+            try:
+                delattr(request.cfg.cache, attrname + "2id_lower")
+            except:
+                pass
+        else:
+            setattr(request.cfg.cache, attrname + "2id", cache[attrname])
+            setattr(request.cfg.cache, attrname + "2id_lower", cache[attrname + "_lower"])
+
+
+def loadLookupCaches(request):
+    """load lookup cache contents into memory: cfg.cache.XXX2id"""
+    scope, arena, cachekey = 'userdir', 'users', 'lookup'
+    diskcache = caching.CacheEntry(request, arena, cachekey, scope=scope, use_pickle=True)
+    try:
+        cache = diskcache.content()
+    except caching.CacheError:
+        cache = {}
+        for attrname in CACHED_USER_ATTRS:
+            cache[attrname] = {}
+    cache_with_lowercase = addLowerCaseKeys(cache)
+    setMemoryLookupCaches(request, cache_with_lowercase)
+
+
+def rebuildLookupCaches(request):
+    """complete attrs -> userid lookup cache rebuild"""
+    # as there may be thousands of users and reading all profiles is
+    # expensive, we just have 1 lookup cache for all interesting user attrs,
+    # so we only need to read all profiles ONCE to build the cache.
+    scope, arena, key = 'userdir', 'users', 'lookup'
+    diskcache = caching.CacheEntry(request, arena, key, scope=scope, use_pickle=True, do_locking=False)
+    diskcache.lock('w')
+
+    cache = {}
+    for attrname in CACHED_USER_ATTRS:
+        cache[attrname] = {}
+    for userid in getUserList(request):
+        u = User(request, id=userid)
+        if u.valid:
+            for attrname in CACHED_USER_ATTRS:
+                if hasattr(u, attrname):
+                    attr2id = cache[attrname]
+                    value = getattr(u, attrname)
+                    if isinstance(value, list):
+                        for val in value:
+                            attr2id[val] = userid
+                    else:
+                        attr2id[value] = userid
+
+    cache_with_lowercase = addLowerCaseKeys(cache)
+    setMemoryLookupCaches(request, cache_with_lowercase)
+    diskcache.update(cache)
+    diskcache.unlock()
+    return cache
+
+
+def clearLookupCaches(request):
+    """kill the userid lookup cache"""
+    # this triggers a rebuild of the cache.
+    setMemoryLookupCaches(request, None)
+    scope, arena, key = 'userdir', 'users', 'lookup'
+    caching.CacheEntry(request, arena, key, scope=scope).remove()
+
+
+def addLowerCaseKeys(cache):
+    """add lowercased lookup keys, so we can support case-insensitive lookup"""
+    c = deepcopy(cache)  # we do not want to modify cache itself
+    for attrname in CACHED_USER_ATTRS:
+        attr2id = c[attrname]
+        attr2id_lower = c[attrname + "_lower"] = {}
+        for key, value in attr2id.iteritems():
+            attr2id_lower[key.lower()] = value
+    return c
+
+
 def getUserId(request, searchName):
     """ Get the user ID for a specific user NAME.
 
@@ -134,6 +236,16 @@ def getUserIdByOpenId(request, openid):
     return _getUserIdByKey(request, 'openids', openid)
 
 
+def superusers(request):
+    """
+    yields superuser User objects
+    """
+    for name in request.cfg.superuser:
+        u = User(request, auth_username=name)
+        if u.isSuperUser():  # this checks for addtl. criteria
+            yield u
+
+
 def getUserIdentification(request, username=None):
     """ Return user name or IP or '<unknown>' indicator.
 
@@ -173,7 +285,7 @@ def encodePassword(cfg, pwd, salt=None, scheme=None):
         if salt is None:
             salt = random_string(20)
         assert isinstance(salt, str)
-        hash = hash_new('sha1', pwd)
+        hash = hashlib.new('sha1', pwd)
         hash.update(salt)
         return '{SSHA}' + base64.encodestring(hash.digest() + salt).rstrip()
     else:
@@ -603,13 +715,13 @@ class User:
                     if scheme == '{SSHA}':
                         d = base64.decodestring(d)
                         salt = d[20:]
-                        hash = hash_new('sha1', password.encode('utf-8'))
+                        hash = hashlib.new('sha1', password.encode('utf-8'))
                         hash.update(salt)
                         enc = base64.encodestring(hash.digest() + salt).rstrip()
 
                     elif scheme == '{SHA}':
                         enc = base64.encodestring(
-                            hash_new('sha1', password.encode('utf-8')).digest()).rstrip()
+                            hashlib.new('sha1', password.encode('utf-8')).digest()).rstrip()
 
                     elif scheme == '{APR1}':
                         # d is of the form "$apr1$<salt>$<hash>"
@@ -686,28 +798,19 @@ class User:
             data.write(line + '\n')
         data.close()
 
-        arena = 'user'
-        key = 'name2id'
-        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
-        try:
-            del self._request.cfg.cache.name2id
-        except:
-            pass
-        key = 'openid2id'
-        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
-        try:
-            del self._request.cfg.cache.openid2id
-        except:
-            pass
-
         if not self.disabled:
             self.valid = 1
 
+        self.updateLookupCaches()
+
         if not self._stored:
             self._stored = True
             event = events.UserCreatedEvent(self._request, self)
             events.send_event(event)
 
+        # update page subscriber's cache after saving user preferences
+        self.updatePageSubCache()
+
     # -----------------------------------------------------------------
     # Time and date formatting
 
@@ -804,9 +907,6 @@ class User:
         The subscription list may contain page names or interwiki page
         names. e.g 'Page Name' or 'WikiName:Page_Name'
 
-        TODO: check if it's fast enough when getting called for many
-              users from page.getSubscribersList()
-
         @param pagelist: list of pages to check for subscription
         @rtype: bool
         @return: if user is subscribed any page in pagelist
@@ -895,6 +995,70 @@ class User:
             self.save()
         return not self.isSubscribedTo([pagename])
 
+    def updatePageSubCache(self):
+        """ When a user profile is saved, we update the page subscriber's cache """
+
+        scope, arena, key = 'userdir', 'users', 'pagesubscriptions'
+
+        cache = caching.CacheEntry(self._request, arena=arena, key=key, scope=scope, use_pickle=True, do_locking=False)
+        if not cache.exists():
+            return  # if no cache file exists, just don't do anything
+
+        cache.lock('w')
+        page_sub = cache.content()
+
+        # we only store entries for valid users with some page subscriptions
+        if self.valid and self.subscribed_pages:
+            page_sub[self.id] = {
+                'name': self.name,
+                'email': self.email,
+                'subscribed_pages': self.subscribed_pages,
+            }
+        elif page_sub.get(self.id):
+            del page_sub[self.id]
+
+        cache.update(page_sub)
+        cache.unlock()
+
+    def updateLookupCaches(self):
+        """ When a user profile is saved, we update the userid lookup caches """
+
+        scope, arena, key = 'userdir', 'users', 'lookup'
+
+        diskcache = caching.CacheEntry(self._request, arena=arena, key=key, scope=scope, use_pickle=True, do_locking=False)
+        if not diskcache.exists():
+            return  # if no cache file exists, just don't do anything
+
+        diskcache.lock('w')
+        cache = diskcache.content()
+        userid = self.id
+
+        # first remove all old entries mapping to this userid:
+        for attrname in CACHED_USER_ATTRS:
+            attr2id = cache[attrname]
+            for key, value in attr2id.items():
+                if value == userid:
+                    del attr2id[key]
+
+        # then, if user is valid, update with the current attr values:
+        if self.valid:
+            for attrname in CACHED_USER_ATTRS:
+                if hasattr(self, attrname):
+                    value = getattr(self, attrname)
+                    if value:
+                        # we do not store empty values, likely not unique
+                        attr2id = cache[attrname]
+                        if isinstance(value, list):
+                            for val in value:
+                                attr2id[val] = userid
+                        else:
+                            attr2id[value] = userid
+
+        cache_with_lowercase = addLowerCaseKeys(cache)
+        setMemoryLookupCaches(self._request, cache_with_lowercase)
+        diskcache.update(cache)
+        diskcache.unlock()
+
     # -----------------------------------------------------------------
     # Quicklinks
 
@@ -1096,7 +1260,7 @@ class User:
     def generate_recovery_token(self):
         key = random_string(64, "abcdefghijklmnopqrstuvwxyz0123456789")
         msg = str(int(time.time()))
-        h = hmac_new(key, msg).hexdigest()
+        h = hmac.new(key, msg).hexdigest()
         self.recoverpass_key = key
         self.save()
         return msg + '-' + h
@@ -1109,12 +1273,12 @@ class User:
             stamp = int(parts[0])
         except ValueError:
             return False
-        # only allow it to be valid for twelve hours
-        if stamp + 12*60*60 < time.time():
+        lifetime = self._request.cfg.recovery_token_lifetime * 3600
+        if time.time() > stamp + lifetime:
             return False
         # check hmac
         # key must be of type string
-        h = hmac_new(str(self.recoverpass_key), str(stamp)).hexdigest()
+        h = hmac.new(str(self.recoverpass_key), str(stamp)).hexdigest()
         if not safe_str_equal(h, parts[1]):
             return False
         self.recoverpass_key = ""
